<?php

namespace Drupal\themed_fast_404;

use Drupal\Core\Config\ConfigFactory;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\Core\Url;
use Drupal\file\FileRepositoryInterface;

/**
 * Fast 404 manager service.
 */
class ThemedFast404Manager implements ThemedFast404ManagerInterface {

  /**
   * The file repository.
   *
   * @var \Drupal\file\FileRepositoryInterface
   */
  protected $fileRepository;

  /**
   * The config factory service.
   *
   * @var \Drupal\Core\Config\ConfigFactory
   */
  protected $configFactory;

  /**
   * The language manager.
   *
   * @var \Drupal\Core\Language\LanguageManagerInterface
   */
  protected $languageManager;

  /**
   * Themed fast 404 constructor.
   *
   * @param \Drupal\file\FileRepositoryInterface $fileRepository
   *   File repository.
   * @param \Drupal\Core\Config\ConfigFactory $config_factory
   *   The config factory service.
   * @param \Drupal\Core\Language\LanguageManagerInterface $language_manager
   *   The language manager.
   */
  public function __construct(FileRepositoryInterface $fileRepository, ConfigFactory $config_factory, LanguageManagerInterface $language_manager) {
    $this->fileRepository = $fileRepository;
    $this->configFactory = $config_factory;
    $this->languageManager = $language_manager;
  }

  /**
   * {@inheritDoc}
   */
  public function buildStatic404() {
    // Get base_url form config.
    $base_url = $this->configFactory->get('themed_fast_404.settings')
      ->get('base_url');

    foreach ($this->languageManager->getLanguages() as $language) {
      // Generate Drupal 404 absolute url.
      $path = Url::fromRoute(static::DRUPAL_404_ROUTE_NAME, [], ['language' => $language]);
      $url = $base_url ? $base_url . $path->toString() : $path->setAbsolute()->toString();

      // Get Drupal 404 page contents.
      $html = file_get_contents($url);
      $destination = static::PAGE_NOT_FOUND_FILE_PATH . 'page-not-found-' . $language->getId() . '.html';
      $this->fileRepository->writeData($html, $destination, FileSystemInterface::EXISTS_REPLACE);
    }
  }

}
